package com.bsoft.api.common.handlers;

import com.bsoft.api.common.Result;
import com.bsoft.api.common.exceptions.DBConfigurationError;
import com.bsoft.api.common.exceptions.ExceptionBase;
import com.bsoft.api.common.exceptions.InvalidTokenException;
import com.bsoft.common.utils.HttpUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;

/**
 * 全局异常处理类
 */
@ControllerAdvice
public class GlobalExceptionHandler {
    private static final Logger log = LoggerFactory.getLogger(GlobalExceptionHandler.class);

    /**
     * 其他异常处理
     * @param request
     * @param e
     * @return
     */
    @ExceptionHandler(Exception.class)
    @ResponseBody
    public Object defaultErrorHandler(HttpServletRequest request, Exception e){
        String url = HttpUtil.getIP(request);
        log.error(url + "请求未知异常:" + e.getMessage(), e.getStackTrace());
        return Result.error();
    }

    /**
     * 其他内部异常
     * @param request
     * @param e
     * @return
     */
    @ExceptionHandler(ExceptionBase.class)
    @ResponseBody
    public Object BaseErrorHandler(HttpServletRequest request, Exception e){
        String url = HttpUtil.getIP(request);
        log.error(url + "请求未知异常:" + e.getMessage(), e.getStackTrace());
        return Result.error();
    }

    @ExceptionHandler(DBConfigurationError.class)
    @ResponseBody
    public Object DBConfigurationErrorHandler(HttpServletRequest request, Exception e){
        String url = HttpUtil.getIP(request);
        log.error(url + "请求未知异常:" + e.getMessage(), e.getStackTrace());
        return Result.error(400, e.getMessage());
    }

    /**
     * 无效token
     * @param request
     * @param e
     * @return
     */
    @ExceptionHandler(InvalidTokenException.class)
    @ResponseBody
    public Object InvalidTokenExceptionHandler(HttpServletRequest request, Exception e){
        String url = HttpUtil.getIP(request);
        log.error(url + "请求未知异常:" + e.getMessage(), e.getStackTrace());
        return Result.error(Result.ErrorCode.INVALID_TOKEN);
    }
}
