package com.bsoft.api.service.Impl;

import com.bsoft.api.common.exceptions.DBConfigurationError;
import com.bsoft.api.mapper.BlockValuesMapper;
import com.bsoft.api.mapper.DicDimMapper;
import com.bsoft.api.mapper.SerPageBlockRsMapper;
import com.bsoft.api.mapper.SerPageMapper;
import com.bsoft.api.model.DicDim;
import com.bsoft.api.model.SerPage;
import com.bsoft.api.model.SerPageBlockRs;
import com.bsoft.api.service.AsynBlockValuesService;
import com.bsoft.api.service.BlockValuesService;
import org.apache.ibatis.binding.MapperMethod;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

@Service
public class BlockValuesServiceImpl implements BlockValuesService {

    @Resource
    private SerPageBlockRsMapper serPageBlockRsMapper;
    @Resource
    private SerPageMapper serPageMapper;
    @Resource
    private AsynBlockValuesService asynBlockValuesServiceImpl;


    @Override
    public List<Map<String, Object>> getBlockValuesByPageCode(Integer pageCode, Integer disease, Integer doctor, Integer department, Integer time) throws InterruptedException {
        /**
         * 1.根据pageCode,disease,doctor,department,time查询所有对应的pageId
         *   查询要求为pageCode为当前pageCode，相同pageId的所有数据有且仅有DIM_ID为
         *   disease,doctor,department,time中不为null的对应id
         * 2.根据pageId获取所有block
         * 3.根据block拼接表名及where语句
         */
        List<Map<String, Object>> list = new ArrayList<>();

        int count = 0;
        String inField = "";
        String whereClause = "where 1=1";
        if(disease != null){
            whereClause += String.format(" and disease='%s'", disease);
            inField += "'disease',";
            count++;
        }
        if (doctor != null) {
            whereClause += String.format(" and doctor='%s'", doctor);
            inField += "'doctor',";
            count++;
        }
        if (department != null) {
            whereClause += String.format(" and department='%s'", department);
            inField += "'department',";
            count++;
        }
        if (time != null) {
            whereClause += String.format(" and time='%s'", time);
            inField += "'time',";
            count++;
        }
        if(count > 0){
            inField = inField.substring(0, inField.length()-1);
        }

        List<SerPage> pageList = serPageMapper.selectByCodeAndDim(pageCode, inField, count);
        if(pageList.size() == 0){
            throw new DBConfigurationError("页面维度配置表错误，请联系管理员");
        }

        SerPage page = pageList.get(0);

        //根据pageId查询所有板块
        List<SerPageBlockRs> pageBlocklist = serPageBlockRsMapper.selectByPageId(page.getId());
        //循环获取数据
        CountDownLatch latch = new CountDownLatch(pageBlocklist.size());
        for (SerPageBlockRs data : pageBlocklist) {
            //获取表名
            String blockId = data.getBlockId().toString();
            String tableName = "VAL_BLOCK_VALUES_" + blockId;
            asynBlockValuesServiceImpl.getBlockValues(list, blockId, tableName, whereClause, latch);

        }
        latch.await();

        return list;
    }
}
